#!/bin/bash
#
# movehtml:bin/movehtml-api.sh
#
declare -r MOVE_HTML_CONFIG_FILE=~/.config/movehtml

function expandPath() {
  local    path=
  local -a pathElements=()
  local -a resultPathElements=()

  IFS=':' read -r -a pathElements <<<"$1"
  : "${pathElements[@]}"
  for path in "${pathElements[@]}"; do
    : "$path"
    case $path in
      "~+"/*)
        path=$PWD/${path#"~+/"}
        ;;
      "~-"/*)
        path=$OLDPWD/${path#"~-/"}
        ;;
      "~"/*)
        path=$HOME/${path#"~/"}
        ;;
      "~"*)
        username=${path%%/*}
        username=${username#"~"}
        # shellcheck disable=SC2162
        IFS=: read _ _ _ _ _ homedir _ < <(getent passwd "$username")
        if [[ $path = */* ]]; then
          path=${homedir}/${path#*/}
        else
          path=$homedir
        fi
        ;;
    esac
    resultPathElements+=( "$path" )
  done
  local result
  printf -v result '%s:' "${resultPathElements[@]}"
  printf '%s\n' "${result%:}"
}

function move_html_all {
  local -r source_folder="$1"
  local -r destination_folder="$2"
  local -r continue_on_error="${3:-false}"

  if [ -z "${source_folder}" ]; then
    echofatal 'Parameter source folder is missing.'
    return 100
  fi
  if [ ! -d "${source_folder}" ]; then
    echofatal "Cannot find source folder '${source_folder}'."
    return 100
  fi

  if [ -z "${destination_folder}" ]; then
    echofatal 'Parameter destination folder is missing.'
    return 100
  fi
  if [ ! -d "${destination_folder}" ]; then
    echofatal "Cannot find destination folder '${destination_folder}'."
    return 100
  fi

  pushd "${source_folder}" >/dev/null || return $?

  local current_html_file=
  local current_html_folder=
  local -i rc=
  find . -maxdepth 1 -type f -name '*.html' -print0 | while IFS= read -r -d $'\0' current_html_file; do
    current_html_folder="${current_html_file%.html}_files"

    echo "Expected HTML file: '${current_html_file}'"
    echo "Expected HTML dir : '${current_html_folder}'"
    do_move_html "${current_html_file}" "${current_html_folder}" "${destination_folder}"
    rc=$?
    if [ "${rc}" -ne 0 ] && [ "${continue_on_error}" != 'true' ] ; then
      return "${rc}"
    fi
  done

  # shellcheck disable=SC2181
  if [ $? -eq 0 ] ; then
    find . -maxdepth 1 -type d -name "*_files" -print0 | while IFS= read -r -d $'\0' current_html_folder; do
      echoerror "Warning directory: '${current_html_folder}' exist but no html file."
    done
  fi

  popd >/dev/null || return $?
}

function do_move_html {
  local -r html_file="$1"
  local -r html_folder="$2"
  local -r destination_folder="$3"

  if [ ! -f "${html_file}" ]; then
    echofatal "Cannot find html file '${html_file}'."
    return 100
  fi
  if [ ! -d "${html_folder}" ]; then
    echofatal "Cannot find html folder '${html_folder}'."
    return 100
  fi
  if [ ! -d "${destination_folder}" ]; then
    echofatal "Cannot find destination '${destination_folder}'."
    return 100
  fi

  mv -v "${html_file}" "${destination_folder}" || return $?
  local destfullname=
  destfullname="${destination_folder}/$( basename "${html_file}" )" || return $?
  if [ ! -f "${destfullname}" ]; then
    echofatal "Cannot find moved html file '${destfullname}'."
    return 100
  fi

  mv "${html_folder}" "${destination_folder}" || return $?
  destfullname="${destination_folder}/$( basename "${html_folder}" )" || return $?
  if [ ! -d "${destfullname}" ]; then
    echofatal "Cannot find moved html folder '${destfullname}'."
    return 100
  fi
}

function __get_config_file_value_v0__ {
  local -r name="$1"

  if [ ! -f "${MOVE_HTML_CONFIG_FILE}" ]; then
    echowarn "MOVE_HTML_CONFIG_FILE='${MOVE_HTML_CONFIG_FILE}' not found."
  else
    local value=
    value="$( grep "${name}" "${MOVE_HTML_CONFIG_FILE}" | cut -d '=' -f2 )" || true
    if [ -z "${value}" ] ; then
      echowarn "Can't find '${name}' in '${MOVE_HTML_CONFIG_FILE}'."
    else
      echo "${value}"
    fi
  fi
}

function get_config_file_value {
  local -r name="$1"

  if [ ! -f "${MOVE_HTML_CONFIG_FILE}" ]; then
    echowarn "MOVE_HTML_CONFIG_FILE='${MOVE_HTML_CONFIG_FILE}' not found." >&2
    return 100
  else
    local value_json=
    if ! value_json="$( jq ".[\"${name}\"]" "${MOVE_HTML_CONFIG_FILE}" )" ; then
      echowarn "${FUNCNAME[0]}() - Try to use old configuration format for '${name}'"
      __get_config_file_value_v0__ "${name}" || return $?
    elif [ "${value_json}" = 'null' ] ; then
      echowarn "${FUNCNAME[0]}() - .[\"${name}\"]=${value_json}"
    else
       jq -r ".[\"${name}\"]" "${MOVE_HTML_CONFIG_FILE}" || return $?
    fi
  fi
}

function __config_migration_txt2json_v0__ {
  local index=
  index="$( __get_config_file_value_v0__ 'index' )" || true

  local source_folder=
  source_folder="$( __get_config_file_value_v0__ 'source_folder' )" || true

  local destination_folder=
  destination_folder="$( __get_config_file_value_v0__ 'destination_folder' )" || true

  cat <<EOF
{
  "version": 1,
  "index": ${index},
  "source_folder": "${source_folder}",
  "destination_folder": "${destination_folder}"
}
EOF
}

function set_config_file_value {
  local -r name="$1"
  local -r value="$2"

  if [ ! -f "${MOVE_HTML_CONFIG_FILE}" ]; then
    cat <<EOF >"${MOVE_HTML_CONFIG_FILE}"
{
  "version": 1
}
EOF
  fi

  local json_content=
  if ! json_content="$( jq -c '.' "${MOVE_HTML_CONFIG_FILE}" )" ; then
    echowarn "${FUNCNAME[0]}() - Try to use old configuration format for '${name}'"
    json_content="$( __config_migration_txt2json_v0__ )" || return $?
  fi

  jq -c ". | .[\"${name}\"]|=\"${value}\"" <<<"${json_content}" >"${MOVE_HTML_CONFIG_FILE}" || return $?
}

function inc_index {
  local -i index='0'

  if index="$( get_config_file_value 'index' )" ; then
    index="$((index + 1))"
  else
    echowarn "'index' property not found in configuration file - assume 0."
  fi

  set_config_file_value 'index' "${index}" || return $?

  echo "${index}"
}

function move_html_build_destination_folder {
 local -r destination_base="$1"

 local index=
 index="$( inc_index )" || return $?

 echo "${destination_base}${index}"
}

function get_default_source_folder {
  get_config_file_value 'source_folder' || return $?
}

function set_default_source_folder {
  set_config_file_value 'source_folder' "$1" || return $?
}

function get_default_destination_folder_base {
  get_config_file_value 'destination_folder' || return $?
}

function set_default_destination_folder_base {
  set_config_file_value 'destination_folder' "$1" || return $?
}
