#!/bin/bash
#
# movehtml:bin/movehtml-config.sh
#
# usage:
#  movehtml-config
#  movehtml-config "${SOURCE_FOLDER}" "${DESTINATION_FOLDER}"
#
set -u
set -o pipefail

PROJECT_PATH="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_PATH

# shellcheck disable=SC1090
source "${PROJECT_PATH}/apis/bash-common-api/bash-common-api.sh" || exit $?
# shellcheck disable=SC1090
source "${PROJECT_PATH}/bin/movehtml-api.sh" || exit $?

declare -r __PRODUCT_NAME__='movehtml'
declare -r __PRODUCT_COMMENT__='Configure default values'
declare -r BACKTITLE="${__PRODUCT_NAME__} === ${__PRODUCT_COMMENT__} === "

function usage {
  cat <<EOF
Usage: $0

  $( basename "$0" ) --help
  $( basename "$0" ) [--source-dir SOURCE] [--destination-dir-base DEST]

Actions:
  -h,           --help      Display help and exit.

Parameters:
  -S SOURCE,    --source-dir SOURCE
                            Set default source directory if missing ask for value.
  -D DEST,      --destination-dir-base DEST
                            Set default destination directory base if missing ask for value.

EOF
}

function is_folder_or_parent_exist {
  local -r folder="$1"

  if [ -d "${folder}" ] ; then
    echookay "Directory '${folder}' exists. OK" >&2
    echo 'true'
    return 0
  fi

  local parent=
  parent="$( dirname "${folder}" )" || return $?

  if [ -d "${parent}" ] ; then
    echookay "Directory '${folder}' not found - Found '${parent}' has parent directory of '${folder}'" >&2
    echo 'true'
  else
    echowarn "Directory '${folder}' not found - Can't find directory for '${folder}' and event for '${parent}'"
    echo 'false'
  fi
}

function display_summary {
  cat <<EOF | jq '.' || return $?
{
  "source": {
    "directory": "${param_source_folder}",
    "exist": $( is_folder_or_parent_exist "${param_source_folder}" )
  },
  "destination": {
    "directory": "${param_destination_folder_base}",
    "exist": $( is_folder_or_parent_exist "${param_destination_folder_base}" )
  }
}
EOF
}

function main {
  local param=
  local param_destination_folder_base=
  local param_source_folder=

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage
        exit 0
        ;;

      '-S'|'--source-dir')
        if ! param_source_folder="$( P "$@" )" ; then
          echofatal "Value required for '${param}'"
          usage >&2
          exit 1
        fi
        shift
        ;;

      '-D'|'--destination-dir-base')
        if ! param_destination_folder_base="$( P "$@" )" ; then
          echofatal "Value required for '${param}'"
          usage >&2
          exit 1
        fi
        shift
        ;;

      *)
        echofatal "Unknown option: '${param}'"
        usage >&2
        exit 1
        ;;
    esac
  done

  if [ -z "${param_source_folder}" ]; then
    param_source_folder="$( get_default_source_folder )" || return $?

    local source=
    source="$(
        whiptail --backtitle "${BACKTITLE}" --inputbox 'Give default source folder' \
            10 50 "${param_source_folder}" 3>&1 1>&2 2>&3
        )"
    case $? in
      0) param_source_folder="${source}" ;; # OK
      1) ;; # Cancel (ignore new value)
      255|*) # Exit
        echofatal 'Cancel by user'
        return 100
        ;;
    esac
  fi

  if [ -z "${param_destination_folder_base}" ]; then
    param_destination_folder_base="$( get_default_destination_folder_base )"

    local destination=
    destination="$(
        whiptail --backtitle "${BACKTITLE}" --inputbox 'Give default destination folder base' \
            10 50 "${param_destination_folder_base}" 3>&1 1>&2 2>&3
        )"
    case $? in
      0) param_destination_folder_base="${destination}" ;; # OK
      1) ;; # Cancel (ignore new value)
      255|*) # Exit
        echofatal 'Cancel by user'
        return 100
        ;;
    esac
  fi

  param_source_folder="$( expandPath "${param_source_folder}" )" || return $?
  param_destination_folder_base="$( expandPath "${param_destination_folder_base}" )" || return $?

  display_summary || return $?

  set_default_source_folder           "${param_source_folder}" || return $?
  set_default_destination_folder_base "${param_destination_folder_base}" || return $?

  jq '.' "${MOVE_HTML_CONFIG_FILE}" >&2
}

main "$@" || exit $?
